package edu.kit.ipd.milton.algorithms;

import java.util.LinkedList;
import java.util.List;

import edu.kit.ipd.milton.timeseries.TimeSeries;
import edu.kit.ipd.milton.timeseries.TimeSeriesPoint;

/**
 * 
 * @author aenglhardt
 *
 */
public class Aggregator {

	public static double[][] aggregate(double[][] ts, int aggregation) {
		if (ts.length == 0)
			return new double[0][];
		if (aggregation == 0)
			return ts;

		final int newLength = (int) Math.ceil((double) ts.length / aggregation);
		final double[][] aggTS = new double[newLength][2];
		int i = 0;
		for (int j = 0; j < newLength; j++) {
			aggTS[j][0] = ts[i][0];
			for (int agg = 0; agg < aggregation && i < ts.length; agg++) {
				aggTS[j][1] += ts[i++][1];
			}
		}
		return aggTS;
	}

	public static double[] aggregate(double[] ts, int aggregation) {
		if (ts.length == 0)
			return new double[0];
		if (aggregation == 0)
			return ts;

		final int newLength = (int) Math.ceil((double) ts.length / aggregation);
		final double[] aggTS = new double[newLength];
		int i = 0;
		for (int j = 0; j < newLength; j++) {
			for (int agg = 0; agg < aggregation && i < ts.length; agg++) {
				aggTS[j] += ts[i++];
			}
		}
		return aggTS;
	}

	public static TimeSeries aggregte(TimeSeries ts, int aggregation) {
		if (ts.getPoints().isEmpty())
			return new TimeSeries(new LinkedList<TimeSeriesPoint>());
		if (aggregation == 0)
			return ts;

		final List<TimeSeriesPoint> tsPoints = ts.getPoints();
		final int newLength = (int) Math.ceil((double) ts.size() / aggregation);
		final List<TimeSeriesPoint> aggTS = new LinkedList<>();
		int i = 0;
		TimeSeriesPoint curVal;
		for (int j = 0; j < newLength; j++) {
			curVal = tsPoints.get(i);
			for (int agg = 0; agg < aggregation && i < tsPoints.size(); agg++) {
				curVal = curVal.addPoint(tsPoints.get(i++));
			}
			aggTS.add(curVal);
		}
		return new TimeSeries(aggTS);

	}

	private Aggregator() {

	}
}
