package edu.kit.ipd.milton.events;

import java.util.LinkedList;
import java.util.List;

import edu.kit.ipd.milton.util.Tuple;

/**
 * Contains sets of PE, MISS and FP events of two time series
 * 
 * @author aenglhardt
 *
 */
public class AssociatedEventCollection {

	private final List<Tuple<Event>> pairedEvents;
	private final List<Event> missingEvents;
	private final List<Event> falsePositiveEvents;

	public AssociatedEventCollection() {
		this.pairedEvents = new LinkedList<>();
		this.missingEvents = new LinkedList<>();
		this.falsePositiveEvents = new LinkedList<>();
	}

	public AssociatedEventCollection(List<Tuple<Event>> pairedEvents, List<Event> missingEvents,
			List<Event> falsePositiveEvents) {
		this.pairedEvents = pairedEvents;
		this.missingEvents = missingEvents;
		this.falsePositiveEvents = falsePositiveEvents;
	}

	public void buildFromMissingChanges(List<Event> firstEvents, List<Event> secondEvents) {
		if (firstEvents.size() == 0) {
			if (secondEvents.size() != 0) {
				for (Event c : secondEvents)
					falsePositiveEvents.add(c);
			}
		} else {
			if (secondEvents.size() == 0) {
				for (Event c : firstEvents)
					missingEvents.add(c);
			}
		}
	}

	public final List<Tuple<Event>> getPairedEvents() {
		return pairedEvents;
	}

	public final List<Event> getMissingEvents() {
		return missingEvents;
	}

	public final List<Event> getFalsePositiveEvents() {
		return falsePositiveEvents;
	}

	@Override
	public String toString() {
		String s;
		s = "Paired Events:\n";
		s += pairedEvents.toString() + "\n";
		s += "Missing Events:\n";
		s += missingEvents.toString() + "\n";
		s += "False positive Events:\n";
		s += falsePositiveEvents.toString() + "\n";
		return s;
	}
}
