package edu.kit.ipd.milton.events;

import java.util.LinkedList;
import java.util.List;

import edu.kit.ipd.milton.measure.Error;
import edu.kit.ipd.milton.timeseries.TimeSeries;
import edu.kit.ipd.milton.timeseries.TimeSeriesPoint;

/**
 * Change consisting of a timestamp and a score
 * 
 * @author aenglhardt
 *
 */
public class Change implements Event {
	public static final EventType TYPE = EventType.CHANGE;
	public static final String cpExt = ".cp";

	private final double timestamp;
	private final double score;

	public Change(double timestamp, double score) {
		this.timestamp = timestamp;
		this.score = score;
	}

	public double getTimestamp() {
		return timestamp;
	}

	public double getScore() {
		return score;
	}

	@Override
	public double calcDistance(Event other) {
		return Math.abs(timestamp - ((Change) other).getTimestamp());
	}

	public static List<Event> calculateFromTimeSeries(TimeSeries ts) {
		List<Event> changes = new LinkedList<>();
		for (TimeSeriesPoint tsp : ts.getPoints()) {
			changes.add(new Change(tsp.getTime(), tsp.getValue()));
		}
		return changes;
	}

	@Override
	public List<Event> cloneEventList(List<Event> original) {
		List<Event> cloned = new LinkedList<>();
		for (Event c : original)
			cloned.add(c.clone());
		return cloned;
	}

	@Override
	public double visitError(Error e) {
		return e.calcSingularError(this);
	}

	@Override
	public double visitError(Error e, Event other) {
		return e.calcPairedError(this, (Change) other);
	}

	@Override
	public Change clone() {
		return new Change(timestamp, score);
	}

	@Override
	public String toString() {
		return String.format("|%8.4f, %8.4f|", Double.valueOf(timestamp), Double.valueOf(score));
	}

	@Override
	public EventType getType() {
		return TYPE;
	}

}
