package edu.kit.ipd.milton.events;

import java.util.LinkedList;
import java.util.List;

import edu.kit.ipd.milton.util.StdStats;
import edu.kit.ipd.milton.util.Tuple;

/**
 * Normalization of score and time with algorithm 4 and 5 of thesis for score
 * and time
 * 
 * @author aenglhardt
 *
 */
public class Normalization {

	public static final double EPSILON = 0.00000001;

	private Normalization() {

	}

	public static Tuple<List<Pattern>> normalizeLocationAndScorePatternsOnly(List<Pattern> a, List<Pattern> b,
			double epsilon) {
		double minLocation = Double.MAX_VALUE;
		double maxLocation = Double.MIN_VALUE;
		double minScore = Double.MAX_VALUE;
		double maxScore = Double.MIN_VALUE;
		double curMinScore = 0;
		double curMaxScore = 0;
		double curLocation = 0;

		for (Pattern p : a) {
			curMinScore = StdStats.min(p.getEventSeries());
			curMaxScore = StdStats.max(p.getEventSeries());
			curLocation = p.getLocation();
			if (curMinScore < minScore)
				minScore = curMinScore;
			if (curMaxScore > maxScore)
				maxScore = curMaxScore;
			if (curLocation < minLocation)
				minLocation = curLocation;
			if (curLocation > maxLocation)
				maxLocation = curLocation;
		}
		for (Pattern p : b) {
			curMinScore = StdStats.min(p.getEventSeries());
			curMaxScore = StdStats.max(p.getEventSeries());
			curLocation = p.getLocation();
			if (curMinScore < minScore)
				minScore = curMinScore;
			if (curMaxScore > maxScore)
				maxScore = curMaxScore;
			if (curLocation < minLocation)
				minLocation = curLocation;
			if (curLocation > maxLocation)
				maxLocation = curLocation;
		}

		double distanceLocation = maxLocation + epsilon - minLocation;
		double distanceScore = maxScore + epsilon - minScore;

		final List<Pattern> normalizedPatternsOriginal = applyPatternNormalization(a, minLocation, distanceLocation,
				minScore, distanceScore, epsilon);
		final List<Pattern> normalizedPatternsChanged = applyPatternNormalization(b, minLocation, distanceLocation,
				minScore, distanceScore, epsilon);

		return new Tuple<>(normalizedPatternsOriginal, normalizedPatternsChanged);
	}

	public static Tuple<List<Pattern>> normalizeScorePatternsOnly(List<Pattern> a, List<Pattern> b, double epsilon) {
		double minScore = Double.MAX_VALUE;
		double maxScore = Double.MIN_VALUE;
		double curMinScore = 0;
		double curMaxScore = 0;

		for (Pattern p : a) {
			curMinScore = StdStats.min(p.getEventSeries());
			curMaxScore = StdStats.max(p.getEventSeries());
			if (curMinScore < minScore)
				minScore = curMinScore;
			if (curMaxScore > maxScore)
				maxScore = curMaxScore;
		}
		for (Pattern p : b) {
			curMinScore = StdStats.min(p.getEventSeries());
			curMaxScore = StdStats.max(p.getEventSeries());
			if (curMinScore < minScore)
				minScore = curMinScore;
			if (curMaxScore > maxScore)
				maxScore = curMaxScore;
		}

		double distanceScore = maxScore + epsilon - minScore;

		final List<Pattern> normalizedPatternsOriginal = applyScoresPatternNormalization(a, curMinScore, distanceScore,
				epsilon);
		final List<Pattern> normalizedPatternsChanged = applyScoresPatternNormalization(b, curMinScore, distanceScore,
				epsilon);

		return new Tuple<>(normalizedPatternsOriginal, normalizedPatternsChanged);
	}

	public static Tuple<List<Pattern>> normalizeLocationPatternsOnly(List<Pattern> a, List<Pattern> b, double epsilon) {
		if (a == null || b == null) {
			throw new RuntimeException("To normalize lists are null");
		}
		double minLocation = Double.MAX_VALUE;
		double maxLocation = Double.MIN_VALUE;
		double curLocation = 0;

		for (Pattern p : a) {
			curLocation = p.getLocation();

			if (curLocation < minLocation)
				minLocation = curLocation;
			if (curLocation > maxLocation)
				maxLocation = curLocation;
		}
		for (Pattern p : b) {
			curLocation = p.getLocation();
			if (curLocation < minLocation)
				minLocation = curLocation;
			if (curLocation > maxLocation)
				maxLocation = curLocation;
		}

		double distanceLocation = maxLocation + epsilon - minLocation;

		final List<Pattern> normalizedPatternsOriginal = applyLocationPatternNormalization(a, minLocation,
				distanceLocation, epsilon);
		final List<Pattern> normalizedPatternsChanged = applyLocationPatternNormalization(a, minLocation,
				distanceLocation, epsilon);

		return new Tuple<>(normalizedPatternsOriginal, normalizedPatternsChanged);
	}

	private static List<Pattern> applyPatternNormalization(List<Pattern> original, double minLocation,
			double distanceLocation, double minScore, double distanceScore, double epsilon) {
		final List<Pattern> normalizedPatterns = new LinkedList<>();
		double newLocation = 0;
		double newScore = 0;
		for (Pattern p : original) {
			newLocation = (p.getLocation() + epsilon - minLocation) / distanceLocation;
			double[] prevScores = p.getEventSeries();
			double[] newScores = new double[prevScores.length];
			for (int i = 0; i < newScores.length; i++) {
				newScore = (prevScores[i] - minScore) / distanceScore;
				newScores[i] = newScore;
				if (newScore > 1 || newScore < 0)
					System.out.println("newScore = " + newScore);
			}
			if (newLocation > 1 || newLocation < 0)
				System.out.println("newTime = " + newLocation);
			normalizedPatterns.add(new Pattern(newLocation, newScores));
		}
		return normalizedPatterns;
	}

	private static List<Pattern> applyScoresPatternNormalization(List<Pattern> original, double minScore,
			double distanceScore, double epsilon) {
		final List<Pattern> normalizedPatterns = new LinkedList<>();
		double newScore = 0;
		for (Pattern p : original) {
			double[] prevScores = p.getEventSeries();
			double[] newScores = new double[prevScores.length];
			for (int i = 0; i < newScores.length; i++) {
				newScore = (prevScores[i] - minScore) / distanceScore;
				newScores[i] = newScore;
				if (newScore > 1 || newScore < 0)
					System.out.println("newScore = " + newScore);
			}
			normalizedPatterns.add(new Pattern(p.getLocation(), newScores));
		}
		return normalizedPatterns;
	}

	private static List<Pattern> applyLocationPatternNormalization(List<Pattern> original, double minLocation,
			double distanceLocation, double epsilon) {
		final List<Pattern> normalizedPatterns = new LinkedList<>();
		double newLocation = 0;
		for (Pattern p : original) {
			newLocation = (p.getLocation() + epsilon - minLocation) / distanceLocation;
			if (newLocation > 1 || newLocation < 0)
				System.out.println("newTime = " + newLocation);
			normalizedPatterns.add(new Pattern(newLocation, p.getEventSeries()));
		}
		return normalizedPatterns;
	}

	public static List<Pattern> normalizeScoreTimeSeriesPatternsWithInterval(List<Pattern> patterns, double a,
			double b, double epsilon) {
		double distance = b + epsilon - a;
		List<Pattern> normalizedPatterns = new LinkedList<>();
		if (distance > 0) {
			normalizedPatterns = applyScoresPatternNormalization(patterns, a, distance, epsilon);
		}
		return normalizedPatterns;
	}

	public static List<Pattern> normalizeLocationTimeSeriesPatternsWithInterval(List<Pattern> patterns, double a,
			double b, double epsilon) {
		double distance = b + epsilon - a;
		List<Pattern> normalizedPatterns = new LinkedList<>();
		if (distance > 0) {
			normalizedPatterns = applyLocationPatternNormalization(patterns, a, distance, epsilon);
		}
		return normalizedPatterns;
	}

	public static Tuple<List<Change>> normalizeScoreTimeSeriesOnlyOnChanges(List<Change> a, List<Change> b,
			double epsilon) {
		double minScore = Double.MAX_VALUE;
		double maxScore = Double.MIN_VALUE;
		double curScore = 0;

		for (Change c : a) {
			curScore = c.getScore();
			if (curScore < minScore)
				minScore = curScore;
			if (curScore > maxScore)
				maxScore = curScore;
		}
		for (Change c : b) {
			curScore = c.getScore();
			if (curScore < minScore)
				minScore = curScore;
			if (curScore > maxScore)
				maxScore = curScore;
		}

		double distance = maxScore + epsilon - minScore;

		final List<Change> normalizedChangesOriginal = new LinkedList<>();
		final List<Change> normalizedChangesChanged = new LinkedList<>();
		if (distance > 0) {
			double newScore = 0;
			for (Change c : a) {
				newScore = (c.getScore() + epsilon - minScore) / distance;
				normalizedChangesOriginal.add(new Change(c.getTimestamp(), newScore));
				if (newScore > 1 || newScore < 0)
					System.out.println("newScore = " + newScore);
			}
			for (Change c : b) {
				newScore = (c.getScore() + epsilon - minScore) / distance;
				normalizedChangesChanged.add(new Change(c.getTimestamp(), newScore));
				if (newScore > 1 || newScore < 0)
					System.out.println("newScore = " + newScore);
			}
		}

		return new Tuple<>(normalizedChangesOriginal, normalizedChangesChanged);
	}

	public static Tuple<List<Change>> normalizeTimeTimeSeriesOnlyOnChanges(List<Change> a, List<Change> b,
			double epsilon) {
		double minTime = Double.MAX_VALUE;
		double maxTime = Double.MIN_VALUE;
		double curTime = 0;

		for (Change c : a) {
			curTime = c.getTimestamp();
			if (curTime < minTime)
				minTime = curTime;
			if (curTime > maxTime)
				maxTime = curTime;
		}
		for (Change c : b) {
			curTime = c.getTimestamp();
			if (curTime < minTime)
				minTime = curTime;
			if (curTime > maxTime)
				maxTime = curTime;
		}

		double distance = maxTime + epsilon - minTime;

		final List<Change> normalizedChangesOriginal = new LinkedList<>();
		final List<Change> normalizedChangesChanged = new LinkedList<>();
		if (distance > 0) {
			double newTime = 0;
			for (Change c : a) {
				newTime = (c.getTimestamp() + epsilon - minTime) / distance;
				normalizedChangesOriginal.add(new Change(newTime, c.getScore()));
				if (newTime > 1 || newTime < 0)
					System.out.println("newTime = " + newTime);
			}
			for (Change c : b) {
				newTime = (c.getTimestamp() + epsilon - minTime) / distance;
				normalizedChangesChanged.add(new Change(newTime, c.getScore()));
				if (newTime > 1 || newTime < 0)
					System.out.println("newTime = " + newTime);
			}
		}

		return new Tuple<>(normalizedChangesOriginal, normalizedChangesChanged);
	}

	public static List<Change> normalizeScoreTimeSeriesWithChangeInterval(List<Change> changes, double a, double b,
			double epsilon) {
		double distance = b + epsilon - a;

		final List<Change> normalizedChanges = new LinkedList<>();
		if (distance > 0) {
			double newScore = 0;
			for (Change c : changes) {
				newScore = (c.getScore() + epsilon - a) / distance;
				normalizedChanges.add(new Change(c.getTimestamp(), newScore));
				if (newScore > 1 || newScore < 0)
					System.out.println("newScore = " + newScore);
			}
		}

		return normalizedChanges;
	}

	public static List<Change> normalizeTimeTimeSeriesWithChangeInterval(List<Change> changes, double a, double b,
			double epsilon) {
		double distance = b + epsilon - a;

		final List<Change> normalizedChanges = new LinkedList<>();
		if (distance > 0) {
			double newTime = 0;
			for (Change c : changes) {
				newTime = (c.getTimestamp() + epsilon - a) / distance;
				normalizedChanges.add(new Change(newTime, c.getScore()));
				if (newTime > 1 || newTime < 0)
					System.out.println("newTime = " + newTime);
			}
		}

		return normalizedChanges;
	}
}
