package edu.kit.ipd.milton.events;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import edu.kit.ipd.milton.measure.Error;
import edu.kit.ipd.milton.timeseries.TimeSeries;
import edu.kit.ipd.milton.timeseries.TimeSeriesPoint;

/**
 * 
 * @author aenglhardt
 *
 */
public class Pattern implements Event {
	public static final EventType TYPE = EventType.PATTERN;
	public static final String patExt = ".pats";

	private final double location;
	private final double[] eventSeries;

	public Pattern(double location, double[] eventSeries) {
		if (location < 0 || eventSeries == null || eventSeries.length < 1) {
			throw new IllegalArgumentException();
		}
		this.location = location;
		this.eventSeries = eventSeries;
	}

	public Pattern(double[] timeSeries, int index, int length) {
		if (timeSeries == null || index < 0 || length < 0) {
			throw new IllegalArgumentException();
		}

		if (index + length > timeSeries.length) {
			length = timeSeries.length - index - 1;
		}
		this.location = index;
		this.eventSeries = new double[length];

		for (int i = 0; i < eventSeries.length; i++) {
			eventSeries[i] = timeSeries[index + i];
		}
	}

	public double getLocation() {
		return location;
	}

	public double[] getEventSeries() {
		return eventSeries;
	}

	@Override
	public double calcDistance(Event other) {
		return Math.abs(location - ((Pattern) other).getLocation());
	}

	public static List<Event> calculateFromTimeSeries(TimeSeries ts) {
		List<Event> patterns = new LinkedList<>();
		for (TimeSeriesPoint tsp : ts.getPoints()) {
			patterns.add(new Pattern(tsp.getTime(), tsp.getMeasurements()));
		}
		return patterns;
	}

	@Override
	public List<Event> cloneEventList(List<Event> original) {
		List<Event> cloned = new LinkedList<>();
		for (Event c : original)
			cloned.add(c.clone());
		return cloned;
	}

	@Override
	public double visitError(Error e) {
		return e.calcSingularError(this);
	}

	@Override
	public double visitError(Error e, Event other) {
		return e.calcPairedError(this, (Pattern) other);
	}

	@Override
	public Pattern clone() {
		return new Pattern(location, Arrays.copyOf(eventSeries, eventSeries.length));
	}

	@Override
	public String toString() {
		return "Event [location=" + location + ", eventSeries=" + Arrays.toString(eventSeries) + "]";
	}

	@Override
	public EventType getType() {
		return TYPE;
	}

}
