package edu.kit.ipd.milton.exporter;

import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import edu.kit.ipd.milton.measure.MeasureValue;

/**
 * Sums up split errors with given parameter
 * 
 * @author aenglhardt
 *
 */
public class BasicExporter {
	private final List<String> params;
	private final Map<String, Double> timeError;
	private final Map<String, Double> scoreError;
	private final Map<String, Double> pcError;
	private final Map<String, Double> missError;
	private final Map<String, Double> fpError;
	private final Map<String, Double> totalError;
	private int numFiles;

	public BasicExporter() {
		this.params = new LinkedList<>();
		this.timeError = new HashMap<>();
		this.scoreError = new HashMap<>();
		this.pcError = new HashMap<>();
		this.missError = new HashMap<>();
		this.fpError = new HashMap<>();
		this.totalError = new HashMap<>();
		this.numFiles = 0;
	}

	public void addNewError(String param, MeasureValue error) {
		Double newTimeError;
		Double newScoreError;
		Double newPcError;
		Double newMissError;
		Double newFpError;
		Double newTotError;
		if (!params.contains(param)) {
			params.add(param);
			newTimeError = Double.valueOf(error.getTimeError());
			newScoreError = Double.valueOf(error.getScoreError());
			newPcError = Double.valueOf(error.getPcError());
			newMissError = Double.valueOf(error.getMissError());
			newFpError = Double.valueOf(error.getFpError());
			newTotError = Double.valueOf(error.totalError());

		} else {
			newTimeError = Double.valueOf(timeError.get(param).doubleValue() + error.getTimeError());
			newScoreError = Double.valueOf(scoreError.get(param).doubleValue() + error.getScoreError());
			newPcError = Double.valueOf(pcError.get(param).doubleValue() + error.getPcError());
			newMissError = Double.valueOf(missError.get(param).doubleValue() + error.getMissError());
			newFpError = Double.valueOf(fpError.get(param).doubleValue() + error.getFpError());
			newTotError = Double.valueOf(totalError.get(param).doubleValue() + error.totalError());
		}
		timeError.put(param, newTimeError);
		scoreError.put(param, newScoreError);
		pcError.put(param, newPcError);
		missError.put(param, newMissError);
		fpError.put(param, newFpError);
		totalError.put(param, newTotError);
		numFiles++;
	}

	public String logTotalError() {
		String s = "";
		Collections.sort(params);
		s += "Num Files = " + (numFiles / params.size()) + "\n";
		for (String curParam : params) {
			s += String.format("%s;%8.8f\n", curParam, totalError.get(curParam));
		}
		return s;
	}

	public String logErrorsSplit() {
		String s = "";
		Collections.sort(params);
		s += "Num Files = " + (numFiles / params.size()) + "\n";
		s += "Time Error\n";
		for (String curParam : params) {
			s += String.format("%s;%8.8f\n", curParam, timeError.get(curParam));
		}
		s += "Score Error\n";
		for (String curParam : params) {
			s += String.format("%s;%8.8f\n", curParam, scoreError.get(curParam));
		}
		s += "PC Error\n";
		for (String curParam : params) {
			s += String.format("%s;%8.8f\n", curParam, pcError.get(curParam));
		}
		s += "Miss Error\n";
		for (String curParam : params) {
			s += String.format("%s;%8.8f\n", curParam, missError.get(curParam));
		}
		s += "FP Error\n";
		for (String curParam : params) {
			s += String.format("%s;%8.8f\n", curParam, fpError.get(curParam));
		}
		s += "Total Error\n";
		for (String curParam : params) {
			s += String.format("%s;%8.8f\n", curParam, totalError.get(curParam));
		}
		return s;
	}

	public String logErrors() {
		String s = "";
		Collections.sort(params);
		s += "Num Files = " + (numFiles / params.size()) + "\n";
		for (String curParam : params)
			s += String.format("%s;%8.8f;%8.8f;%8.8f;%8.8f;%8.8f;%8.8f\n", curParam, timeError.get(curParam),
					scoreError.get(curParam), pcError.get(curParam), missError.get(curParam), fpError.get(curParam),
					totalError.get(curParam));
		return s;
	}
}
