package edu.kit.ipd.milton.exporter;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.FileUtils;

import edu.kit.ipd.milton.measure.MeasureValue;

/**
 * Sums up errors for different scenarios with given scenario keys.
 * 
 * @author aenglhardt
 *
 */
public class ScenarioExporter {
	private final String exportPath;
	private final List<String> scenarios;
	private final Map<String, BasicExporter> values;

	public ScenarioExporter(String exportPath) {
		this.exportPath = exportPath;
		this.scenarios = new LinkedList<String>();
		this.values = new HashMap<>();
	}

	public void addNewError(String curScenario, String param, MeasureValue error) {
		if (!scenarios.contains(curScenario)) {
			scenarios.add(curScenario);
			values.put(curScenario, new BasicExporter());
		}
		values.get(curScenario).addNewError(param, error);
	}

	public String logErrors(String folder) {
		String s = "";
		Collections.sort(scenarios);
		for (String curScenario : scenarios) {
			s += curScenario + "\n";
			s += values.get(curScenario).logTotalError();
			try {
				final String fileName = exportPath + folder + "_" + curScenario + ".err";
				FileUtils.writeStringToFile(new File(fileName), values.get(curScenario).logErrors());
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return s;
	}
}
