package edu.kit.ipd.milton.functions;

import java.util.List;

/**
 * Piecewise function with k functions and k - 1 cut points where the functions
 * changes in ascending order
 * 
 * @author aenglhardt
 *
 */
public class PiecewiseFunction implements Function {

	private final List<Function> fs;
	private final List<Double> cutPoints;

	public PiecewiseFunction(List<Function> fs, List<Double> cutPoints) {
		if (fs == null || cutPoints == null)
			throw new IllegalArgumentException("Function list or cut point list is null");

		if (fs.size() != cutPoints.size() + 1)
			throw new IllegalArgumentException("Not enough functions for number of cut points");

		this.fs = fs;
		this.cutPoints = cutPoints;
	}

	@Override
	public double calc(double x) {
		int i = 0;
		while (cutPoints.size() > i && x > cutPoints.get(i).doubleValue()) {
			i++;
		}
		return fs.get(i).calc(x);
	}

}
