package edu.kit.ipd.milton.measure;

import java.util.List;

import edu.kit.ipd.milton.events.AssociatedEventCollection;
import edu.kit.ipd.milton.events.Change;
import edu.kit.ipd.milton.events.Event;
import edu.kit.ipd.milton.events.Pattern;
import edu.kit.ipd.milton.functions.Function;
import edu.kit.ipd.milton.util.StdStats;

/**
 * errMISS
 * 
 * @author aenglhardt
 *
 */
public class ErrorMISS implements Error {

	private final Function f;
	private int numMiss;

	public ErrorMISS(Function f) {
		this.f = f;
		this.numMiss = 0;
	}

	public int getNumMiss() {
		return numMiss;
	}

	@Override
	public double calcError(AssociatedEventCollection changes) {
		double error = 0;
		List<Event> misses = changes.getMissingEvents();
		this.numMiss = misses.size();
		for (Event e : misses) {
			final double curError = e.visitError(this);
			error += curError;
		}
		return error;
	}

	@Override
	public double calcSingularError(Change c) {
		return f.calc(c.getScore());
	}

	@Override
	public double calcSingularError(Pattern p) {
		return f.calc(StdStats.sum(p.getEventSeries()));
	}

	@Override
	public double calcPairedError(Change a, Change b) {
		throw new IllegalAccessError("MISS not callable for tuple");
	}

	@Override
	public double calcPairedError(Pattern a, Pattern b) {
		throw new IllegalAccessError("MISS not callable for tuple");
	}
}
