package edu.kit.ipd.milton.measure;

import java.util.List;

import edu.kit.ipd.milton.events.AssociatedEventCollection;
import edu.kit.ipd.milton.events.Change;
import edu.kit.ipd.milton.events.Event;
import edu.kit.ipd.milton.events.Pattern;
import edu.kit.ipd.milton.functions.Function;
import edu.kit.ipd.milton.util.StdStats;
import edu.kit.ipd.milton.util.Tuple;

/**
 * errScore
 * 
 * @author aenglhardt
 *
 */
public class ErrorSCORE implements Error {
	private final Function f;
	private int numPE;

	public ErrorSCORE(Function f) {
		this.f = f;
		this.numPE = 0;
	}

	public Function getFunction() {
		return f;
	}

	public int getNumPE() {
		return numPE;
	}

	@Override
	public double calcError(AssociatedEventCollection changes) {
		double error = 0;
		List<Tuple<Event>> pairedEvents = changes.getPairedEvents();
		this.numPE = pairedEvents.size();
		for (Tuple<Event> pe : pairedEvents) {
			final double curError = pe.getA().visitError(this, pe.getB());
			error += curError;
		}
		return error;
	}

	@Override
	public double calcSingularError(Change c) {
		throw new IllegalAccessError("SCORE not callable for single Event");
	}

	@Override
	public double calcSingularError(Pattern p) {
		throw new IllegalAccessError("SCORE not callable for single Event");
	}

	@Override
	public double calcPairedError(Change a, Change b) {
		final double deltaS = a.getScore() - b.getScore();
		return f.calc(deltaS);
	}

	@Override
	public double calcPairedError(Pattern a, Pattern b) {
		return f.calc(StdStats.calculateEuclideanDistance(a.getEventSeries(), b.getEventSeries()));
	}
}
