package edu.kit.ipd.milton.measure;

import edu.kit.ipd.milton.events.AssociatedEventCollection;

/**
 * Measures calculated from a {@link AssociatedEventCollection}
 * 
 * @author aenglhardt
 *
 */
public class Measure {
	private final ErrorTIME errorTIME;
	private final ErrorSCORE errorSCORE;
	private final ErrorMISS errorMISS;
	private final ErrorFP errorFP;

	public Measure(ErrorTIME errorTIME, ErrorSCORE errorSCORE, ErrorMISS errorMISS, ErrorFP errorFP) {
		this.errorTIME = errorTIME;
		this.errorSCORE = errorSCORE;
		this.errorMISS = errorMISS;
		this.errorFP = errorFP;
	}

	public ErrorTIME getErrorTIME() {
		return errorTIME;
	}

	public ErrorSCORE getErrorSCORE() {
		return errorSCORE;
	}

	public ErrorMISS getErrorMISS() {
		return errorMISS;
	}

	public ErrorFP getErrorFP() {
		return errorFP;
	}

	public MeasureValue calcMeasure(AssociatedEventCollection changeCol) {
		final double errTIME = errorTIME.calcError(changeCol);
		final double errSCORE = errorSCORE.calcError(changeCol);
		final double errMISS = errorMISS.calcError(changeCol);
		final double errFP = errorFP.calcError(changeCol);
		if (errorTIME.getNumPE() != errorSCORE.getNumPE())
			throw new RuntimeException("Num PC differs in errTIME and errSCORE");
		final int numPC = errorTIME.getNumPE();
		final int numMISS = errorMISS.getNumMiss();
		final int numFP = errorFP.getNumFP();
		return new MeasureValue(errTIME, errSCORE, errMISS, errFP, numPC, numMISS, numFP);
	}
}
