package edu.kit.ipd.milton.measure;

/**
 * Measure consisting of errTIME, errSCORE, errMISS and errFP
 * 
 * @author aenglhardt
 *
 */
public class MeasureValue {
	public static final String NUMBER_PRINT_FORMAT = "%8.4f";

	private final double errTIME;
	private final double errSCORE;
	private final double errMISS;
	private final double fpError;
	private final int numOrigChanges;
	private final int numChangedChanges;
	private final int numPC;
	private final int numMiss;
	private final int numFP;

	public MeasureValue(double timeError, double scoreError, double missError, double fpError, int numPC, int numMISS,
			int numFP) {
		this.errTIME = timeError;
		this.errSCORE = scoreError;
		this.errMISS = missError;
		this.fpError = fpError;
		this.numOrigChanges = numPC + numMISS;
		this.numChangedChanges = numPC + numFP;
		this.numPC = numPC;
		this.numMiss = numMISS;
		this.numFP = numFP;
	}

	public double getTimeError() {
		return errTIME;
	}

	public double getScoreError() {
		return errSCORE;
	}

	public double getPcError() {
		return errTIME + errSCORE;
	}

	public double getMissError() {
		return errMISS;
	}

	public double getFpError() {
		return fpError;
	}

	public int getNumOrigChanges() {
		return numOrigChanges;
	}

	public int getNumChangedChanges() {
		return numChangedChanges;
	}

	public int getNumPC() {
		return numPC;
	}

	public int getNumMiss() {
		return numMiss;
	}

	public int getNumFP() {
		return numFP;
	}

	public double totalError() {
		return (errTIME + errSCORE + errMISS + fpError) / (numOrigChanges + 1);
	}

	public String csvExport() {
		final String csv_format = "%.6f";
		String numChanges = numOrigChanges + ";" + numChangedChanges + ";" + numPC + ";" + numMiss + ";" + numFP;
		String errors = String.format(csv_format + ";" + csv_format + ";" + csv_format + ";" + csv_format + ";"
				+ csv_format + ";" + csv_format, Double.valueOf(errTIME), Double.valueOf(errSCORE),
				Double.valueOf(getPcError()), Double.valueOf(errMISS), Double.valueOf(fpError),
				Double.valueOf(totalError()));
		return numChanges + ";" + errors;
	}

	@Override
	public String toString() {
		return String.format("timeError   = " + NUMBER_PRINT_FORMAT + "\nscoreError  = " + NUMBER_PRINT_FORMAT
				+ "\npcError     = " + NUMBER_PRINT_FORMAT + "\n" + "missError   = " + NUMBER_PRINT_FORMAT
				+ "\nfpError     = " + NUMBER_PRINT_FORMAT + "\nTotal Error = " + NUMBER_PRINT_FORMAT,
				Double.valueOf(errTIME), Double.valueOf(errSCORE), Double.valueOf(getPcError()),
				Double.valueOf(errMISS), Double.valueOf(fpError), Double.valueOf(totalError()));
	}

	public static MeasureValue defaultError() {
		return new MeasureValue(0, 0, 0, 0, 0, 0, 0);
	}
}
