package edu.kit.ipd.milton.starter;

import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 
 * @author aenglhardt
 *
 */
public class StarterUtil {

	public static List<String> getSubdirectories(String path) {
		final File folder = new File(path);
		if (!folder.isDirectory())
			throw new RuntimeException(path + " not a folder");

		FilenameFilter filter = new FilenameFilter() {
			@Override
			public boolean accept(File arg0, String arg1) {
				return arg0.isDirectory();
			}
		};

		String[] files = folder.list(filter);
		if (files == null) {
			System.out.println("Either dir does not exist or is not a directory");
			return new ArrayList<String>();
		} else {
			return Arrays.asList(files);
		}
	}

	public static List<String> findOtherFiles(String path, final String originalFileName, final String fileExt) {
		final File folder = new File(path);
		if (!folder.isDirectory())
			throw new RuntimeException(path + " not a folder");

		FilenameFilter filter = new FilenameFilter() {
			@Override
			public boolean accept(File arg0, String arg1) {
				return arg1.endsWith(fileExt) && !arg1.equals(originalFileName);
			}
		};

		String[] files = folder.list(filter);
		if (files == null) {
			System.out.println("Either dir does not exist or is not a directory");
			return new ArrayList<String>();
		} else {
			return Arrays.asList(files);
		}
	}

	public static List<String> findOtherFilesWithName(String path, final String originalFileName, final String fileExt) {
		final File folder = new File(path);
		if (!folder.isDirectory())
			throw new RuntimeException(path + " not a folder");

		FilenameFilter filter = new FilenameFilter() {
			@Override
			public boolean accept(File arg0, String arg1) {
				return arg1.endsWith(fileExt) && arg1.contains(originalFileName)
						&& !arg1.equals(originalFileName + fileExt);
			}
		};

		String[] files = folder.list(filter);
		if (files == null) {
			System.out.println("Either dir does not exist or is not a directory");
			return new ArrayList<String>();
		} else {
			return Arrays.asList(files);
		}
	}

	public static List<String> findFilesWithToken(String path, final String[] token, final String fileExt) {
		final File folder = new File(path);
		if (!folder.isDirectory())
			throw new RuntimeException(path + " not a folder");

		FilenameFilter filter = new FilenameFilter() {
			@Override
			public boolean accept(File arg0, String arg1) {
				for (String s : token) {
					if (!arg1.contains(s))
						return false;
				}
				return arg1.endsWith(fileExt);
			}
		};

		String[] files = folder.list(filter);
		if (files == null) {
			System.out.println("Either dir does not exist or is not a directory");
			return new ArrayList<String>();
		} else {
			return Arrays.asList(files);
		}
	}

	public static List<String> findFilesWithAndWithoutToken(String path, final String[] withToken,
			final String[] withoutToken, final String fileExt) {
		final File folder = new File(path);
		if (!folder.isDirectory())
			throw new RuntimeException(path + " not a folder");

		FilenameFilter filter = new FilenameFilter() {
			@Override
			public boolean accept(File arg0, String arg1) {
				for (String s : withToken) {
					if (!arg1.contains(s))
						return false;
				}
				for (String s : withoutToken) {
					if (arg1.contains(s))
						return false;
				}
				return arg1.endsWith(fileExt);
			}
		};

		String[] files = folder.list(filter);
		if (files == null) {
			System.out.println("Either dir does not exist or is not a directory");
			return new ArrayList<String>();
		} else {
			return Arrays.asList(files);
		}
	}

	public static List<String> findFilesWithoutToken(String path, final String[] token, final String fileExt) {
		final File folder = new File(path);
		if (!folder.isDirectory())
			throw new RuntimeException(path + " not a folder");

		FilenameFilter filter = new FilenameFilter() {
			@Override
			public boolean accept(File arg0, String arg1) {
				for (String s : token) {
					if (arg1.contains(s))
						return false;
				}
				return arg1.endsWith(fileExt);
			}
		};

		String[] files = folder.list(filter);
		if (files == null) {
			System.out.println("Either dir does not exist or is not a directory");
			return new ArrayList<String>();
		} else {
			return Arrays.asList(files);
		}
	}

	private StarterUtil() {

	}
}
