package edu.kit.ipd.milton.starter.changedetection;

import java.util.List;

import edu.kit.ipd.milton.algorithms.changedetection.ChangeDetection;
import edu.kit.ipd.milton.events.Change;
import edu.kit.ipd.milton.starter.StarterUtil;
import edu.kit.ipd.milton.util.ReadWriteHelper;

/**
 * 
 * @author aenglhardt
 *
 */
public class RunChangeDetection {

	private static final String EXT = ".csv";

	public static void performChangeDetection(String path, String fileName, ChangeDetection cd, String exportPath) {
		double[][] ts = ReadWriteHelper.readCSVTimeSeries(path + fileName);
		cd.setNewParam(ts);
		List<Change> changes = cd.detectChanges(ts);
		ReadWriteHelper.writeCPFile(exportPath, fileName, changes);
	}

	public static void performChangeDetectionNoParamsSet(String path, String fileName, ChangeDetection cd,
			String exportPath) {
		double[][] ts = ReadWriteHelper.readCSVTimeSeries(path + fileName);
		List<Change> changes = cd.detectChanges(ts);
		ReadWriteHelper.writeCPFile(exportPath, fileName, changes);
	}

	public static void batchChangeDetection(String path, String[] modifyTokens, ChangeDetection cd, String exportPath) {
		List<String> mainFiles = StarterUtil.findFilesWithoutToken(path, modifyTokens, EXT);
		for (String curMainFile : mainFiles) {
			performChangeDetection(path, curMainFile, cd, exportPath);
			List<String> otherFiles = StarterUtil.findOtherFilesWithName(path, curMainFile.replace(EXT, ""), EXT);
			for (String curOtherFile : otherFiles)
				performChangeDetectionNoParamsSet(path, curOtherFile, cd, exportPath);
		}
	}

	public static void batchChangeDetection(String mainPath, String modPath, String[] modifyTokens, ChangeDetection cd,
			String exportPath) {
		List<String> mainFiles = StarterUtil.findFilesWithoutToken(mainPath, modifyTokens, EXT);
		for (String curMainFile : mainFiles) {
			performChangeDetection(mainPath, curMainFile, cd, exportPath);
			List<String> otherFiles = StarterUtil.findOtherFilesWithName(modPath, curMainFile.replace(EXT, ""), EXT);
			for (String curOtherFile : otherFiles)
				performChangeDetectionNoParamsSet(modPath, curOtherFile, cd, exportPath);
		}
	}

	private RunChangeDetection() {

	}
}
