package edu.kit.ipd.milton.starter.simple;

import java.io.File;
import java.util.Locale;

import edu.kit.ipd.milton.Milton;
import edu.kit.ipd.milton.algorithms.changedetection.ChangeDetection;
import edu.kit.ipd.milton.algorithms.changedetection.Cusum;
import edu.kit.ipd.milton.events.EventType;
import edu.kit.ipd.milton.measure.Measure;
import edu.kit.ipd.milton.measure.MeasureValue;
import edu.kit.ipd.milton.starter.changedetection.RunChangeDetection;
import edu.kit.ipd.milton.util.DefaultParameters;
import edu.kit.ipd.milton.util.IntervalNormalizationOption;
import edu.kit.ipd.milton.util.Tuple;

/**
 * 
 * @author aenglhardt
 *
 */
public class ExampleStarter {

	public static final double warpWin = -1;
	public static final double epsModifier = 0.3;
	public static final String cpExt = ".cp";

	public static void main(String[] args) {
		Locale locale = new Locale("en", "US");
		Locale.setDefault(locale);
		runScenario("example", DefaultParameters.standardFinalMeasure());
	}

	private static void runScenario(String name, Measure measure) {
		final String path = "../data/";
		final String fileNameOriginal = "Original";
		final String fileNameChanged = "Changed";

		// Perfrom change detection
		ChangeDetection cd = new Cusum(epsModifier);
		new File(path + "cd/").mkdirs();
		RunChangeDetection.performChangeDetection(path, fileNameOriginal + ".csv", cd, path + "cd/");
		RunChangeDetection.performChangeDetectionNoParamsSet(path, fileNameChanged + ".csv", cd, path + "cd/");

		// Calculate measure
		final Tuple<Double> interval = new Tuple<Double>(Double.valueOf(0), Double.valueOf(1000));
		final double epsilon = 0.0001;
		final IntervalNormalizationOption normOptions = new IntervalNormalizationOption(interval, interval, epsilon);
		final MeasureValue value = Milton.runMeasure(path, "cd/" + fileNameOriginal + cpExt, "cd/" + fileNameChanged
				+ cpExt, measure, EventType.CHANGE, normOptions, warpWin, interval, true);
		Milton.exportMeasure(path, fileNameOriginal, fileNameChanged, value);
	}

	private ExampleStarter() {

	}
}
