package edu.kit.ipd.milton.timeseries;

import java.util.Arrays;

/**
 * 
 * @author aenglhardt
 *
 */
public class TimeSeriesPoint {

	private final double time;
	private final double[] measurements;

	public TimeSeriesPoint(double time, double[] measurements) {
		this.time = time;
		this.measurements = measurements;
	}

	public double getTime() {
		return time;
	}

	public double getValue() {
		if (measurements.length == 1) {
			return measurements[0];
		} else {
			throw new RuntimeException("Value requested for time series point with not exactly one measurement");
		}
	}

	public double[] getMeasurements() {
		return measurements;
	}

	/**
	 * Add another point. Time is original one and a new {@link TimeSeriesPoint }
	 * is returned.
	 * 
	 * @param other
	 * @return
	 */
	public TimeSeriesPoint addPoint(TimeSeriesPoint other) {
		if (other.measurements.length != this.measurements.length) {
			throw new RuntimeException();
		}

		double[] newArray = Arrays.copyOf(measurements, measurements.length);
		for (int i = 0; i < newArray.length; i++) {
			newArray[i] += other.measurements[i];
		}

		return new TimeSeriesPoint(time, newArray);
	}

	@Override
	public String toString() {
		return "|" + time + ", " + Arrays.toString(measurements) + "|";
	}

}
