package edu.kit.ipd.milton.util;

import java.util.LinkedList;
import java.util.List;

import edu.kit.ipd.milton.functions.AbsFunction;
import edu.kit.ipd.milton.functions.ConstantFunction;
import edu.kit.ipd.milton.functions.Function;
import edu.kit.ipd.milton.functions.LinearFunction;
import edu.kit.ipd.milton.functions.PiecewiseFunction;
import edu.kit.ipd.milton.functions.PolynomFunction;
import edu.kit.ipd.milton.measure.ErrorFP;
import edu.kit.ipd.milton.measure.ErrorMISS;
import edu.kit.ipd.milton.measure.ErrorSCORE;
import edu.kit.ipd.milton.measure.ErrorTIME;
import edu.kit.ipd.milton.measure.Measure;

/**
 * Default parameters
 * 
 * @author aenglhardt
 *
 */
public class DefaultParameters {

	private DefaultParameters() {

	}

	/**
	 * 
	 * @return standard parameters
	 */
	public static final Measure standardFinalMeasure() {
		return new Measure(standardErrorTIME(), standardErrorSCORE(), standardErrorMISS(), standardErrorFP());
	}

	public static final ErrorTIME standardErrorTIME() {
		return new ErrorTIME(halfAbsLinearFunction());
	}

	public static final ErrorSCORE standardErrorSCORE() {
		return new ErrorSCORE(halfAbsLinearFunction());
	}

	public static final ErrorMISS standardErrorMISS() {
		return new ErrorMISS(standardLinearFunction());
	}

	public static final ErrorFP standardErrorFP() {
		return new ErrorFP(standardLinearFunction());
	}

	public static final Function sofisticatedFunctionMISSFP() {
		final Function constantFnc = new ConstantFunction(0);
		final Function linearFnc = new LinearFunction(1, 0);

		final List<Function> functions = new LinkedList<>();
		functions.add(constantFnc);
		functions.add(linearFnc);
		final List<Double> cutPoints = new LinkedList<>();
		cutPoints.add(Double.valueOf(0.1));
		final Function pieceWiseFunction = new PiecewiseFunction(functions, cutPoints);

		return pieceWiseFunction;

	}

	public static final Function standardPiecewiseSymFunctionPC() {
		final Function constantFnc = new ConstantFunction(0);
		final Function linearFnc = new LinearFunction(1, 0);

		final List<Function> functions = new LinkedList<>();
		functions.add(new AbsFunction(linearFnc));
		functions.add(constantFnc);
		functions.add(linearFnc);
		final List<Double> cutPoints = new LinkedList<>();
		cutPoints.add(Double.valueOf(-0.001));
		cutPoints.add(Double.valueOf(0.001));
		final Function pieceWiseFunction = new PiecewiseFunction(functions, cutPoints);

		return pieceWiseFunction;
	}

	public static final Function standardPiecewiseSymFunctionPC(double changePoint) {
		final Function constantFnc = new ConstantFunction(0);
		final Function linearFnc = new LinearFunction(1, 0);

		final List<Function> functions = new LinkedList<>();
		functions.add(new AbsFunction(linearFnc));
		functions.add(constantFnc);
		functions.add(linearFnc);
		final List<Double> cutPoints = new LinkedList<>();
		cutPoints.add(Double.valueOf(-changePoint));
		cutPoints.add(Double.valueOf(changePoint));
		final Function pieceWiseFunction = new PiecewiseFunction(functions, cutPoints);

		return pieceWiseFunction;
	}

	public static final Function standardLinearFunction() {
		return new LinearFunction(1, 0);
	}

	public static final Function linearFunction(double a) {
		return new LinearFunction(a, 0);
	}

	public static final Function standardAbsLinearFunction() {
		return new AbsFunction(standardLinearFunction());
	}

	public static final Function halfAbsLinearFunction() {
		return new AbsFunction(linearFunction(0.5));
	}

	public static final Function standardQuadraticFunction() {
		return new PolynomFunction(1, 2, 0);
	}

}
