package edu.kit.ipd.milton.util;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import com.opencsv.CSVReader;
import com.opencsv.CSVWriter;

import edu.kit.ipd.milton.events.Change;
import edu.kit.ipd.milton.events.Pattern;

/**
 * 
 * @author aenglhardt
 *
 */
public class ReadWriteHelper {

	public static final String TIME_FORMAT = "%8.4f";
	public static final String VALUE_FORMAT = "%8.8f";

	private ReadWriteHelper() {

	}

	/**
	 * 
	 * @param fileName
	 *            path and name
	 * @return two dimensional time series with time stamp and value
	 */
	public static double[][] readCSVTimeSeries(String fileName) {
		double[][] timeSeries = null;
		try {
			// Read all
			CSVReader csvReader = new CSVReader(new FileReader(new File(fileName)));
			List<String[]> list = csvReader.readAll();
			String[][] dataArr = new String[list.size()][];
			dataArr = list.toArray(dataArr);

			timeSeries = new double[dataArr.length][];
			double curValue;
			for (int i = 0; i < dataArr.length; i++) {
				if (dataArr[i].length > 0 && !dataArr[i][0].isEmpty()) {
					timeSeries[i] = new double[dataArr[i].length];
					final double timestamp = Double.valueOf(dataArr[i][0]).doubleValue();
					timeSeries[i][0] = timestamp;
					for (int j = 1; j < dataArr[i].length; j++) {
						curValue = Double.valueOf(dataArr[i][j]).doubleValue();
						timeSeries[i][j] = curValue;
					}
				}
			}
			csvReader.close();
		} catch (NumberFormatException | IOException e) {
			e.printStackTrace();
		}
		return timeSeries;
	}

	/**
	 * 
	 * @param fileName
	 *            path and name
	 * @return one dimensional time series with time stamp and value
	 */
	public static double[] readCSVTimeSeriesOneDim(String fileName) {
		double[] timeSeries = null;
		try {
			// Read all
			CSVReader csvReader = new CSVReader(new FileReader(new File(fileName)));
			List<String[]> list = csvReader.readAll();
			String[][] dataArr = new String[list.size()][];
			dataArr = list.toArray(dataArr);

			timeSeries = new double[dataArr.length];
			for (int i = 0; i < dataArr.length; i++) {
				if (dataArr[i].length > 0 && !dataArr[i][1].isEmpty()) {
					final double value = Double.valueOf(dataArr[i][1]).doubleValue();
					timeSeries[i] = value;
				}
			}
			csvReader.close();
		} catch (NumberFormatException | IOException e) {
			e.printStackTrace();
		}
		return timeSeries;
	}

	/**
	 * 
	 * @param path
	 * @param fileName
	 * @param timeSeries
	 *            two dimensional time series with time stamp and value
	 */
	public static void writeCSVTimeSeries(String path, String fileName, double[][] timeSeries) {
		try {
			CSVWriter csvWriter = new CSVWriter(new FileWriter(new File(path + fileName)), ',',
					CSVWriter.NO_QUOTE_CHARACTER, CSVWriter.NO_ESCAPE_CHARACTER);
			final List<String[]> lines = new LinkedList<>();

			for (int i = 0; i < timeSeries.length; i++) {
				final String[] curLine = new String[] { String.format(TIME_FORMAT, Double.valueOf(timeSeries[i][0])),
						String.format(VALUE_FORMAT, Double.valueOf(timeSeries[i][1])) };
				lines.add(curLine);
			}
			csvWriter.writeAll(lines);
			csvWriter.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
	}

	/**
	 * 
	 * @param path
	 * @param fileName
	 * @param timeSeries
	 *            two dimensional time series with time stamp and value
	 */
	public static void writeCSVTimeSeries(String path, String fileName, double[] timeSeries) {
		try {
			CSVWriter csvWriter = new CSVWriter(new FileWriter(new File(path + fileName)), ',',
					CSVWriter.NO_QUOTE_CHARACTER, CSVWriter.NO_ESCAPE_CHARACTER);
			final List<String[]> lines = new LinkedList<>();

			for (int i = 0; i < timeSeries.length; i++) {
				final String[] curLine = new String[] { String.format(VALUE_FORMAT, Double.valueOf(timeSeries[i])) };
				lines.add(curLine);
			}
			csvWriter.writeAll(lines);
			csvWriter.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
	}

	/**
	 * Exports Changes
	 * 
	 * @param path
	 * @param fileName
	 * @param changes
	 */
	public static void writeCPFile(String path, String fileName, List<Change> changes) {
		try {
			CSVWriter csvWriter = new CSVWriter(
					new FileWriter(new File(path + fileName.replace(".csv", Change.cpExt))), ',',
					CSVWriter.NO_QUOTE_CHARACTER, CSVWriter.NO_ESCAPE_CHARACTER);
			final List<String[]> lines = new LinkedList<>();
			for (Change c : changes) {
				final String[] curLine = new String[] { String.format(TIME_FORMAT, Double.valueOf(c.getTimestamp())),
						String.format(VALUE_FORMAT, Double.valueOf(c.getScore())) };
				lines.add(curLine);
			}
			csvWriter.writeAll(lines);
			csvWriter.close();
		} catch (IOException e) {
			throw new RuntimeException("Could not write .cp file");
		}
	}

	/**
	 * Exports Changes
	 * 
	 * @param path
	 * @param fileName
	 * @param patterns
	 */
	public static void writePatsFile(String path, String fileName, List<Pattern> patterns) {
		try {
			CSVWriter csvWriter = new CSVWriter(new FileWriter(
					new File(path + fileName.replace(".csv", Pattern.patExt))), ',', CSVWriter.NO_QUOTE_CHARACTER,
					CSVWriter.NO_ESCAPE_CHARACTER);
			final List<String[]> lines = new LinkedList<>();
			for (Pattern p : patterns) {
				final String[] curLine = new String[p.getEventSeries().length + 1];
				curLine[0] = String.format(TIME_FORMAT, Double.valueOf(p.getLocation()));
				for (int i = 0; i < p.getEventSeries().length; i++) {
					curLine[i + 1] = String.format(VALUE_FORMAT, p.getEventSeries()[i]);
				}
				lines.add(curLine);
			}
			csvWriter.writeAll(lines);
			csvWriter.close();
		} catch (IOException e) {
			throw new RuntimeException("Could not write .pats file");
		}
	}
}
